﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Controls.Grids;

namespace XamGridDynamicCols
{
    public partial class MainPage : UserControl
    {
        #region Private Members

        private ObservableCollection<Dictionary<string, object>> rows;

        #endregion

        #region Public Properties

        public ObservableCollection<Dictionary<string, object>> Rows
        {
            get
            {
                if (this.rows == null)
                    this.rows = new ObservableCollection<Dictionary<string, object>>();

                return this.rows;
            }
        }

        #endregion

        #region Page Constructor
        public MainPage()
        {
            InitializeComponent();
        }
        #endregion

        #region Helper Methods

        /// <summary>
        /// This method will generate random test data for the grid based off of the number of rows and columns desired.
        /// It also has the ability to make a recursive call to itself if child rows are desired.
        /// </summary>
        /// <param name="rowCollection">The collection that will hold the generated data</param>
        /// <param name="numRows">Amount of rows to create</param>
        /// <param name="numCols">Amount of columns to create</param>
        /// <param name="createChildren">Whether to create child rows</param>
        private void GenerateData(ObservableCollection<Dictionary<string, object>> rowCollection, int numRows, int numCols, bool createChildren)
        {
            Random r = new Random((int)DateTime.Now.Ticks);
            for (int i = 0; i < numRows; i++)
            {
                Dictionary<string, object> row = new Dictionary<string, object>();
                for (int j = 0; j < numCols; j++)
                {
                    int randVal = r.Next(500);
                    switch (j % 3)
                    {
                        case 0:
                            row["Col" + j] = "String " + randVal;
                            break;
                        case 1:
                            row["Col" + j] = randVal;
                            break;
                        case 2:
                            row["Col" + j] = randVal % 2 == 1;
                            break;
                    }
                }

                if (createChildren)
                {
                    // Add some child rows
                    ObservableCollection<Dictionary<string, object>> childRows = new ObservableCollection<Dictionary<string, object>>();
                    GenerateData(childRows, r.Next(1, 20), r.Next(1, 10), false);

                    row["Children"] = childRows;
                }

                // Add the row to the collection
                rowCollection.Add(row);
            }
        }

        /// <summary>
        /// This recursive function will add each of the columns from a Dictionary collection to the grid
        /// and will also set up child bands if needed.
        /// </summary>
        /// <param name="columns">The columns collection to add the columns to</param>
        /// <param name="values">
        /// The collection of columns to add to the grid. The value types for the columns are extracted
        /// from this collection.
        /// </param>
        private void AddGridColumns(ColumnBaseCollection columns, Dictionary<string, object> values)
        {
            foreach (string key in values.Keys)
            {

                if (values[key] is string || values[key] is int)
                    // Use a TextColumn for strings and integers
                    columns.Add(new TextColumn { Key = "[" + key + "]", HeaderText = key });
                else if (values[key] is bool)
                    // Use a CheckBoxColumn for booleans
                    columns.Add(new CheckBoxColumn { Key = "[" + key + "]", HeaderText = key, });
                else if (values[key] is ObservableCollection<Dictionary<string, object>>)
                {
                    // If the data type is a collection then we want a ColumnLayout for showing child bands
                    ObservableCollection<Dictionary<string, object>> childRows = values[key] as ObservableCollection<Dictionary<string, object>>;
                    ColumnLayout colLayout = new ColumnLayout { Key = "[" + key + "]" };

                    // Call this function recursively to add to the columns of the ColumnLayout
                    AddGridColumns(colLayout.Columns, childRows[0]);

                    columns.Add(colLayout);
                }
            }
        }

        #endregion

        #region Events

        private void xamGrid1_Loaded(object sender, RoutedEventArgs e)
        {
            xamGrid1.DataContext = Rows;
        }

        private void btnRecreate_Click(object sender, RoutedEventArgs e)
        {
            // Since we're rebinding, clear the collection and clear the columns from the grid, if any
            Rows.Clear();
            xamGrid1.Columns.Clear();

            Random r = new Random();
            // Generate our test data with some child rows as well
            GenerateData(Rows, r.Next(1, 20), r.Next(1, 10), true);

            // Add our generated columns to the grid
            AddGridColumns(xamGrid1.Columns, Rows[0]);
        }

        #endregion
    }
}
